////////////////////////////////////////////////////////////////////////////////
//
//	CHVPicGIF - A C++ class to allow reading and writing of GIF Images
//
//	This reads GIF 87a and 89a, writes only 87a.
//	This writes only 8-bit GIF files. 256 colors.
//	You provide the palette and the palettized image. 
//	This does not do quantization.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef GIFHDRH
#define GIFHDRH

#ifdef HVPICLIB_EXPORTS
#	include "../HVPicLib.h"
#else
#	include "HVPicLib.h"
#endif
#include <Afxtempl.h>

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

typedef struct {
	WORD		ggWidth;
	WORD		ggHeight;
	WORD		ggBitPixel;
	BYTE		ggColorResolution;
	BYTE		ggAspectRatio;
	COLORREF	ggBkColor;
} GIFGLOBAL;


typedef struct {
	RECT		gRect;
	COLORREF	gTransparency; // not -1, transparent is enable.
	BOOL		gbDelayAuto;
	WORD		gDelayTime;
	WORD		gfDisposal;
} GIF89;

typedef CTypedPtrArray<CPtrArray, CHVPicture*> CStillList;

class HVPICLIB_API CHVPicGIF : public CHVPicture {
	CFile*	m_sfp;
	int		m_iGWidth, m_iGHeight;
	BYTE	m_nBkgnd;
	RGBQUAD*m_palette;

	void		(*m_fnSetPixel)( LPBYTE& dst, LPBYTE map, int index );

	GIFGLOBAL	m_global;
	CStillList	m_list;
	COLORREF	m_transparency;

	// temporary var.
	GIF89*		m_gif89;
	LPBYTE		m_codes, m_vals, m_stack, m_sp;
	LPWORD		m_next;
	int			m_curbit, m_lastbit, m_done, m_last_byte,
				m_code_size, m_set_code_size, m_max_code, m_max_code_size, m_firstcode, m_oldcode,
				m_clear_code, m_end_code;

public:
	CHVPicGIF( COLORREF clrTransparency = (DWORD)-1L, BOOL bAutoDelete = TRUE );
	~CHVPicGIF();

	BOOL		Load( CFile& file, UINT uBitPixel = 24 ); // LOAD
	BOOL		Save( CFile& file ); // SAVE

	BOOL		SetSaveOption( int GWidth = -1, int GHeight = -1, BYTE bkgnd = 0, RGBQUAD* palette = NULL );
	CSize		GetDimension( CFile& file );

	BOOL		Attach( HBITMAP bmp, BOOL bAutoDelete = TRUE );
	HBITMAP		Detach();
	BOOL		DeleteObject();

	operator HBITMAP() const;
	operator SIZE() const;

	// special methods
	//
	UINT		GetCount() const;
	GIFGLOBAL	GetHeaderGlobal() const;
	GIF89*		GetHeader( UINT uIndex ) const;
	HBITMAP		GetBitmap( UINT uIndex ) const;
	HBITMAP		Detach( UINT uIndex );

private:
	BOOL			DeleteObject( CStillList& list );

	int				ReadCode( int size, int flag );
	int				ReadLZW( int flag, int size );
	__inline void	ReadExtension( int label );
	__inline LPBYTE	ReadBlock( int* count = NULL, BOOL bSkip = FALSE );
	__inline BOOL	ReadColorTable( int number, LPBYTE& mapbuf );
	BOOL			ReadImage( LPBYTE dst, int width, int height, int bitPixel, LPBYTE map, int interlace );
};


__inline GIFGLOBAL CHVPicGIF::GetHeaderGlobal() const {
	return m_global;
}

#endif